/*
 *	Copyright (c) 1994, 1995 by Jeremy Cooperstock and Steve Kotsopoulos,
 *	All Rights Reserved.
 */

/*
 * cache.c - Cache routines used for publisher buffer.
 *
 * static char rcsid[] = "$Id: cache.c,v 1.13 1998/07/02 19:37:36 steve Exp $";
 */

#include "config.h"
#include "sockutils.h"
#include "afdp.h"
#include "afdplib.h"
#include "afdpsend.h"
#include <sys/stat.h>
#include <sys/fcntl.h>
#ifdef MIPS
#include <sys/file.h>
#define SEEK_SET L_SET
#else
#include <unistd.h>
#include <stdlib.h>
#endif

int fd;
file_info_t file_info;
u_long cache_size;
u_long remaining;
char *cache;
seq_t last_seq;


/* open the file for transmission and fill in the file_info structure */
u_long OpenFile(filename)
char *filename;
{
        struct stat filestats;

	if ((fd = open(filename, O_RDONLY)) == -1) 
		return(-1);

	stat(filename, &filestats);
	remaining = filestats.st_size;
	last_seq = 0;
	strcpy(file_info.name, filename);

	if (command != NULL) 
		strcpy(file_info.command, command);
	else 
		file_info.command[0] = (char) NULL;
	return (remaining);
}

void CreateCache(msg_size) 
u_long msg_size;
{
	cache_size = MIN(msg_size, cache_limit);
	cache = (char *)malloc(cache_size);
	if (cache == NULL) {
		logmsg(LOG_ERR,"malloc failed to allocate %lu bytes",cache_size);
		exit(3);
	}
}

void DestroyCache() 
{
	free (cache);
	close(fd);
}

/* fetch the data portion of a packet */  
int Fetch (buf, seqnum)
char *buf;
u_long seqnum;
{
	static seq_t first_seq = 0;
	u_long readamt, readpos;
	u_long offset, bytes;
	int n;

	/*
	 * If this is the first packet of the message, then stick
 	 * the file info header.  In any case, figure out how many
	 * bytes belong in this packet.
	 */

	if (seqnum == 1) {
	 	memcpy(buf, &file_info, infosize);
		bytes = data_size - infosize;
		buf += infosize;
	}
	else if (seqnum == seq_total)
		bytes = last_data_size;
	else
		bytes = data_size;

	/*
	 * Next, figure out if we have a cache miss.  If so, then
	 * we want to read in the maximum number of bytes that will
	 * fit into the cache but will also fit perfectly in a whole
	 * number of packets (ie. readamt % data_size == 0). However,
	 * we must take into account the effect of the 'infosize' on
	 * the first packet, and also be able to deal with the last
	 * packet, which will in general not use all data_size bytes.
	 *
	 * We must also keep track of the first and last sequence
	 * numbers currently in the cache, in order to detect future
	 * cache misses.  Finally, the number of bytes remaining to
	 * be read in the file is decremented.
	 */
	 
	if (seqnum > last_seq) {
		logmsg(LOG_INFO,"Fetch: cache miss... now refreshing\n");

		if (remaining <= cache_size) 
			readamt = remaining;
		else if (first_seq > 0)
			readamt = (cache_size / data_size) * data_size;
		else
			readamt = ((cache_size + infosize) / data_size) * 
					data_size - infosize;

		readpos = (seqnum-1) * data_size - (seqnum == 1 ? 0 : infosize);

		lseek(fd, readpos, SEEK_SET);
		n = read(fd, cache, readamt);

		if (n != readamt) 
			fatal("Fetch: read error: %m");

		remaining -= readamt;
		first_seq = seqnum;

		if (remaining == 0)
			last_seq = seq_total;
		else if (seqnum == 1) 
			last_seq = first_seq + (readamt+infosize)/data_size - 1;
		else 
			last_seq = first_seq + readamt/data_size - 1;
	}

	/*
	 * If the desired sequence number is below the first sequence 
	 * number currently in the cache, then we are dealing with a
	 * resend request.  Since it doesn't make sense to alter the
	 * cache in this case, we just read the data directly into the
	 * packet.
	 */

	else if (seqnum < first_seq) {
		logmsg(LOG_INFO,"Fetch: cache miss, now retrieving directly\n");
		readpos = (seqnum-1) * data_size - (seqnum == 1 ? 0 : infosize);

		lseek(fd, readpos, SEEK_SET);
		n = read(fd, buf, bytes);

		if (n != bytes) 
			fatal("Fetch: read error: %m");
		return (SUCCESS);
	}

	/*
	 * Calculate how far into the cache we will copy from. 
	 * If the cache currently holds data for packet 1, then
	 * we must take into account 'infosize' bytes when 
	 * calculating the offset of all packets > 1.
	 */

	if (first_seq == 1 && seqnum > 1)
		offset = (seqnum - first_seq) * data_size - infosize;
	else
		offset = (seqnum - first_seq) * data_size; 

	/* now copy relevant data into buffer */
	memcpy(buf, cache + offset, bytes);
	return (SUCCESS);
}
