/*
 *	Copyright (c) 1994, 1995 by Jeremy Cooperstock and Steve Kotsopoulos,
 *	All Rights Reserved.
 */

/*
 * sockutils.h - include file for socket convenience functions
 *
 * rcsid[] = "$Id: sockutils.h,v 1.21 1998/07/02 19:37:36 steve Exp $";
 *
 */

#include "config.h"		/* don't leave home without it */
#include "utils.h"
#undef s			/* defined in utils.h */

/* common-used include files, when dealing with socket calls */
#include <stdio.h>		/* for standard I/O library functions        */
#include <string.h>
#include <sys/types.h>		/* this defines time_t, u_short, u_long etc. */
#include <sys/socket.h>		/* definitions for socket calls		     */
#include <sys/param.h>		/* for MAXHOSTNAMELEN			     */
#include <sys/time.h>
#include <sys/ioctl.h>
#include <netinet/in.h>		/* for in_addr and sockaddr_in structures    */
#include <arpa/inet.h>		/* function definitions for utilities	     */
#include <netdb.h>		/* for hostent structure		     */
#include <net/if.h>		/* used by ConfigureBroadcast() function     */

#if defined (Solaris) || defined (i386)
#define WEIRD_SVR4
#endif

#ifdef WEIRD_SVR4
#include <sys/sockio.h>		/* this is needed for SVR4 on Intel hardware */
#define bzero(b, length) memset ((void *)b, 0, length)
#define bcopy(a, b, length) memcpy ((void *)b, (void *)a, length)
#define getdtablesize() FD_SETSIZE
#else
int getdtablesize();
#endif

#ifndef MIPS
#include <stdlib.h>
#endif

/* external functions and system calls, so that 'gcc -Wall' shuts up */
int bind(), listen(), socket(), select(), sendto(), setsockopt(), atoi();
int close(), connect(), recvfrom(), gethostname(), getpeername();
int accept(), gettimeofday(), unlink(), getopt(), system(), getsockname();
long atol();
#ifndef WEIRD_SVR4
#ifndef __bsdi__
int ioctl();
void bcopy(), bzero();
#endif
#endif

/* functions in sockutils.c */
void CreateSock(), BindSock(), ListenSock(), Fatal();
u_short GetPort(), AssignPort(), PeerPort();
u_long  HostnameToAddr(), Hostaddr(), PeerHost();
void GetPeer(), Usleep();
long Checksum();
int Broadcast(), ReadStream(), WriteStream(), UDPsend(), ConfigureBroadcast();
char *Hostname(), *PeerUser(), *HostaddrToName();

/* global variables, declared in main() */
extern int	debug;

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

#ifndef MAX
#define MAX( a, b )        ((a>b) ? a : b)
#define MIN( a, b )        ((a>b) ? b : a)
#endif

#ifndef INADDR_NONE	/* what planet did you buy your machine from? */
#define INADDR_NONE	0xffffffff
#endif

/* define several levels of debugging messages */

/* we probably don't want more than 1 or 2 DEBUG_0 msgs/packet event   */
/* at this level we should just give general packet header information */
#define DEBUG_0 if (debug) printf	/* for informational messages  */

/* all these levels are probably too detailed, unless you wrote the code */
/* they are used to debug the implementation, and print many lines/pkt   */
#define DEBUG_1 if (debug > 1) printf	/* for debugging messages        */
#define DEBUG_2 if (debug > 2) printf	/* detailed debugging messages   */
#define DEBUG_3 if (debug > 3) printf	/* VERY low-level debugging      */

#define SECOND	1000000			/* in microseconds */
#define mSEC	   1000			/* 1 millisecond   */
